﻿

Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False

    ' =======================================================================================================
    '   APP TITLE AND VERSION
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal Title As String = "") As String
        If Title = "" Then Title = Replace(My.Application.Info.AssemblyName, "_", " ")
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        Return Title & " - V" & s(0) & "." & s(1)
    End Function


    ' =======================================================================================================
    '   Platform selection
    ' =======================================================================================================
    Friend Function CanUseWindowsDlls() As Boolean
        Select Case Environment.OSVersion.Platform
            Case PlatformID.Win32NT, PlatformID.Win32S, PlatformID.Win32Windows, PlatformID.WinCE
                Return True
            Case Else
                Return False
        End Select
    End Function
    Friend Function PlatformAdjustedFileName(ByVal FileName As String, _
                                             Optional ByVal DefaultName As String = "") As String
        If CanUseWindowsDlls() Then
            FileName = Replace(FileName, "/", "\")
            Return FileName
        Else
            If FileName.Contains(":") Then FileName = DefaultName
            FileName = Replace(FileName, "\", "/")
            Return FileName
        End If
    End Function


    ' =======================================================================================
    '  FORM
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
    End Sub

    Private sw As Integer
    Private sh As Integer
    Private Sub GetMaxScreenBounds()
        ' -------------------------------------- Gets an array of all the screens connected to the system.
        Dim Screens() As System.Windows.Forms.Screen = System.Windows.Forms.Screen.AllScreens
        ' -------------------------------------- For each screen
        Dim Index As Integer
        For Index = 0 To Screens.GetUpperBound(0)
            sw = Math.Max(sw, Screens(Index).Bounds.Left + Screens(Index).Bounds.Width)
            sh = Math.Max(sh, Screens(Index).Bounds.Top + Screens(Index).Bounds.Height)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, sw)           ' non oltre le dimensioni di una maximized window
    '        .Height = Math.Min(.Height, sh)         ' non oltre le dimensioni di una maximized window
    '        .Width = Math.Max(.Width, 320)          ' non meno di 320x240
    '        .Height = Math.Max(.Height, 240)        ' non meno di 320x240
    '        .Left = Math.Min(.Left, sw - .Width)    ' non oltre il bordo destro
    '        .Top = Math.Min(.Top, sh - .Height)     ' non oltre il bordo inferiore
    '        .Left = Math.Max(.Left, 0)              ' non meno del bordo sinistro
    '        .Top = Math.Max(.Top, 0)                ' non sopra al bordo superiore
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, sw)               ' not more than the VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, sh)             ' not more than the VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)               ' at least 32x24
            .Height = Math.Max(.Height, 24)             ' at least 32x24
            .Left = Math.Min(.Left, sw - 50)            ' not beyond the right border - 50 pixels
            .Top = Math.Min(.Top, sh - 50)              ' not beyond the bottom border - 50 pixels
            .Left = Math.Max(.Left, 50 - .Width)        ' at least at the left border + 50 pixels
            .Top = Math.Max(.Top, -30)                  ' at least at the top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' ================================================================================================
    '  Combo functions
    ' ================================================================================================
    Friend Sub Combo_InitFromEnum(ByVal combo As ComboBox, ByVal enumType As Type) ' use GeType(enum)
        For Each s As String In [Enum].GetNames(enumType)
            combo.Items.Add(s)
        Next
    End Sub


    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================

    Private Function TabString(ByVal Name As String, _
                              Optional ByVal Value As Double = Double.NaN, _
                              Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt)
        End If
    End Function

    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function

    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function

    Private Function Val_Single(ByVal l As String) As Single
        Return CSng(Val(l.Replace(",", ".")))
    End Function

    Private Function Val_Double(ByVal l As String) As Double
        Return Val(l.Replace(",", "."))
    End Function

    Private Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function

    Private Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function

    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function



    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================

    Friend Sub Save_INI()
        '
        Dim iniFileName As String = PlatformAdjustedFileName(Application.StartupPath & "\" & AssemblyName() & "_INI.txt")
        Dim f As System.IO.StreamWriter
        f = IO.File.CreateText(iniFileName)
        '
        f.WriteLine(" Program Params")
        f.WriteLine("===========================================")
        ' ------------------------------------------------------------------------------ FORM BOUNDS
        Dim r As Rectangle
        r = GetFormRectangle(Form1)
        f.WriteLine(TabString("Form1_Top", r.Top))
        f.WriteLine(TabString("Form1_Left", r.Left))
        f.WriteLine(TabString("Form1_Width", r.Width))
        f.WriteLine(TabString("Form1_Height", r.Height))
        f.WriteLine(TabString("Form1_WindowState", Form1.WindowState))
        '
        '
        f.WriteLine(TabString(""))
        f.WriteLine(TabString(" Slots"))
        f.WriteLine("===========================================")
        f.WriteLine(TabString("Method", Form1.cmb_Method.Text))
        f.WriteLine(TabString("PollingTime", Form1.txt_PollingTime.NumericValueInteger))
        f.WriteLine(TabString("ChangeSpeed", Form1.txt_ChangeSpeed.NumericValueInteger))
        '
        f.WriteLine(TabString(""))
        f.WriteLine(TabString(" Slots"))
        f.WriteLine("===========================================")
        f.WriteLine(TabString("FirstInputSlot", Form1.txt_FirstInputSlot.NumericValueInteger))
        f.WriteLine(TabString("NumInputSlots", Form1.txt_NumInputSlots.NumericValueInteger))
        f.WriteLine(TabString("FirstOutputSlot", Form1.txt_FirstOutputSlot.NumericValueInteger))
        '
        f.Close()
    End Sub



    Friend Sub Load_INI()
        ' ------------------------------------------------------------- 
        'Form1.ThereminoArm_SetDefaultValues()
        ' ------------------------------------------------------------- 
        Dim l As String
        Dim iniFileName As String = PlatformAdjustedFileName(Application.StartupPath & "\" & AssemblyName() & "_INI.txt")
        If My.Computer.FileSystem.FileExists(iniFileName) Then

            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)

            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    Case "Form1_Top" : Form1.Top = Val_Int(l)
                    Case "Form1_Left" : Form1.Left = Val_Int(l)
                        'Case "Form1_Width" : Form1.Width = Val_Int(l)
                        'Case "Form1_Height" : Form1.Height = Val_Int(l)
                    Case "Form1_WindowState" : Form1.WindowState = CType((Val(l)), FormWindowState)
                        ' --------------------------------------------------------------- Params
                    Case "Method" : Form1.cmb_Method.Text = l
                    Case "PollingTime" : Form1.txt_PollingTime.NumericValueInteger = Val_Int(l)
                    Case "ChangeSpeed" : Form1.txt_ChangeSpeed.NumericValueInteger = Val_Int(l)
                        ' --------------------------------------------------------------- Slots
                    Case "FirstInputSlot" : Form1.txt_FirstInputSlot.NumericValueInteger = Val_Int(l)
                    Case "NumInputSlots" : Form1.txt_NumInputSlots.NumericValueInteger = Val_Int(l)
                    Case "FirstOutputSlot" : Form1.txt_FirstOutputSlot.NumericValueInteger = Val_Int(l)
                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form1)
    End Sub

End Module
